package com.git.files.controllers;

import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;

import com.git.files.model.FileInfo;
import com.git.files.model.Tree;
import com.git.files.util.RestFullClient;

import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.http.MediaType;

// REFERENCIAS
// https://docs.gitlab.com/ee/api/repository_files.html
// https://itsallbinary.com/github-rest-api-get-remote-github-repo-files-list-download-file-content-programmatically-without-cloning-in-local/

@Controller
@RequestMapping("/api")
public class FilesController {

  public static final String HOST = "gitlab.com/api/v4";
  public static final String TOKEN = "KKtS27izRvyuMkemR_hN";
  public static final String PROJECT_ID = "28780914";
  public static final String BRANCH = "main";
  public static final String FOLDER = "plantillas";

 @GetMapping(path = "/files/dowload", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
  @ResponseBody
  public ResponseEntity<byte[]> dowloadFile(@RequestParam("file_name") String file_name, @RequestParam("ref") String ref, HttpServletResponse response) throws Exception {

    try {

      // respuesta send headers
      response.addHeader("Content-Disposition", "attachment; filename=" + file_name + "; filename*=UTF-8''" + file_name);

      // send header peticion
      RestFullClient rfc = new RestFullClient();
      String file_path = URLEncoder.encode(file_name, StandardCharsets.UTF_8);
      
      String URI = "https://" + HOST + "/projects/" + PROJECT_ID + "/repository/files/{file_path}/raw?ref={ref}";
      Map<String, String> params = new HashMap<String, String>();
      params.put("ref", ref);
      params.put("file_path", file_path);
      byte[] resp = rfc.dowload(URI, params);
      return ResponseEntity.status(HttpStatus.OK ).body(resp);
    } catch (Exception e) {
      e.printStackTrace();
      return ResponseEntity.status(HttpStatus.EXPECTATION_FAILED).body(null);
    }
 }

  // ******************************************************************************************************
  // LISTAR LOS ARCHIVOS
  // ******************************************************************************************************
  @GetMapping("/files/v2/get")
  @ResponseBody
  public ResponseEntity<Tree[]> getAllByName(@RequestParam("ref") String ref) throws Exception {
    try {
      String URI = "https://" + HOST + "/projects/" + PROJECT_ID + "/repository/tree?ref={ref}";
      RestFullClient rfc = new RestFullClient();
      Map<String, String> params = new HashMap<String, String>();
      params.put("ref", ref);

      Tree[] resp = rfc.getList(URI, params);
      return ResponseEntity.status(HttpStatus.OK ).body(resp);
    } catch (Exception e) {
      e.printStackTrace();
      return ResponseEntity.status(HttpStatus.EXPECTATION_FAILED).body(null);
    }
  }

  // ******************************************************************************************************
  // CREAR EL ARCHIVO
  // ******************************************************************************************************
  @PostMapping("files/v2/store")
  public ResponseEntity<FileInfo> createFilev2(@RequestBody FileInfo file) {
    try {
      String URI = "https://" + HOST + "/projects/" + PROJECT_ID + "/repository/files/" + file.getFile_name();
      RestFullClient rfc = new RestFullClient();
      FileInfo resp = rfc.post(file, URI);
      return ResponseEntity.status(HttpStatus.OK ).body(resp);
    } catch (Exception e) {
      e.printStackTrace();
      return ResponseEntity.status(HttpStatus.EXPECTATION_FAILED).body(null);
    }
  }

  // ******************************************************************************************************
  // ACTUALIZAR EL ARCHIVO
  // ******************************************************************************************************
  @PutMapping("files/v2/update")
  public ResponseEntity<FileInfo> updateFilev2(@RequestBody FileInfo file) {
    try {
      //String file_path = URLEncoder.encode(file.getFile_name(), StandardCharsets.UTF_8);
      String URI = "https://" + HOST + "/projects/" + PROJECT_ID + "/repository/files/" + file.getFile_name();
      RestFullClient rfc = new RestFullClient();
      FileInfo resp = rfc.put(file, URI);
      return ResponseEntity.status(HttpStatus.OK ).body(resp);
    } catch (Exception e) {
      e.printStackTrace();
      return ResponseEntity.status(HttpStatus.EXPECTATION_FAILED).body(null);
    }
  }

  // ******************************************************************************************************
  // ELIMINAR EL ARCHIVO
  // ******************************************************************************************************
  @DeleteMapping("files/v2/delete")
  public ResponseEntity<String> deleteFilev2(@RequestBody FileInfo file) {
    try {
      //String file_path = URLEncoder.encode(file.getFile_name(), StandardCharsets.UTF_8);
      String URI = "https://" + HOST + "/projects/" + PROJECT_ID + "/repository/files/" + file.getFile_name();
      RestFullClient rfc = new RestFullClient();
      rfc.delete(file, URI);
      return ResponseEntity.status(HttpStatus.OK).body("Deleted!");
    } catch (Exception e) {
      e.printStackTrace();
      return ResponseEntity.status(HttpStatus.EXPECTATION_FAILED).body(e.getMessage());
    }
  }

}


