package com.git.files.util;

import java.util.Map;

import com.fasterxml.jackson.annotation.JsonInclude.Include;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.git.files.model.FileInfo;
import com.git.files.model.Tree;
import com.google.gson.Gson;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

public class RestFullClient {

	public static final String TOKEN = "KKtS27izRvyuMkemR_hN";

	RestTemplate restTemplate;

	public RestFullClient() {
		this.restTemplate = new RestTemplate();
	}

	public HttpHeaders createHttpHeaders() {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON);
		headers.setBearerAuth(TOKEN);
		return headers;
	}

	public Tree[] getList(String urlApi, Map<String, String> params) throws JsonProcessingException {
		ResponseEntity<Tree[]> response = restTemplate.exchange(urlApi, HttpMethod.GET,
				new HttpEntity<>(createHttpHeaders()), Tree[].class, params);
		if (response.getStatusCodeValue() == 200 || response.getStatusCodeValue() == 201
				|| response.getStatusCodeValue() == 202) {
			return response.getBody();
		} else {
			return null;
		}
	}

	public byte[] dowload(String urlApi, Map<String, String> params) throws JsonProcessingException {
		ResponseEntity<byte[]> response = restTemplate.exchange(urlApi, HttpMethod.GET,
				new HttpEntity<>(createHttpHeaders()), byte[].class, params);
		if (response.getStatusCodeValue() == 200 || response.getStatusCodeValue() == 201
				|| response.getStatusCodeValue() == 202) {
			return response.getBody();
		} else {
			return null;
		}
	}

	public FileInfo post(FileInfo requestDTO, String urlApi) throws JsonProcessingException {
		ObjectMapper mapper = new ObjectMapper();
		mapper.setSerializationInclusion(Include.NON_NULL);
		String jsonInString = mapper.writeValueAsString(requestDTO);
		ResponseEntity<FileInfo> response = restTemplate.exchange(urlApi, HttpMethod.POST,
				new HttpEntity<>(jsonInString, createHttpHeaders()), FileInfo.class);
		if (response.getStatusCodeValue() == 200 || response.getStatusCodeValue() == 201
				|| response.getStatusCodeValue() == 202) {
			return response.getBody();
		} else {
			return null;
		}
	}

	public FileInfo put(FileInfo requestDTO, String urlApi) throws JsonProcessingException {
		ObjectMapper mapper = new ObjectMapper();
		mapper.setSerializationInclusion(Include.NON_NULL);
		String jsonInString = mapper.writeValueAsString(requestDTO);
		ResponseEntity<FileInfo> response = restTemplate.exchange(urlApi, HttpMethod.PUT,
				new HttpEntity<>(jsonInString, createHttpHeaders()), FileInfo.class);
		if (response.getStatusCodeValue() == 200 || response.getStatusCodeValue() == 201
				|| response.getStatusCodeValue() == 202) {
			return response.getBody();
		} else {
			return new FileInfo();
		}
	}

	public Boolean delete(FileInfo requestDTO, String urlApi) throws JsonProcessingException {
		Gson gson = new Gson();
		String jsonPayload = gson.toJson(requestDTO);
		HttpEntity<String> entity = new HttpEntity<String>(jsonPayload.toString(), createHttpHeaders());
		ResponseEntity<String> response = restTemplate.exchange(urlApi, HttpMethod.DELETE, entity, String.class);
		if (response.getStatusCodeValue() == 204 || response.getStatusCodeValue() == 200
				|| response.getStatusCodeValue() == 201 || response.getStatusCodeValue() == 202) {
			return true;
		} else {
			return false;
		}
	}
}
